package de.qvalentin.rapla2kustom;

import android.app.IntentService;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.util.Log;
import de.qvalentin.rapla2kustom.ical.IcalToStr;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Map;


public class ToKservice extends IntentService {

    public static final String KUSTOM_ACTION = "org.kustom.action.SEND_VAR";
    public static final String KUSTOM_ACTION_EXT_NAME = "org.kustom.action.EXT_NAME";
    public static final String KUSTOM_ACTION_VAR_NAME = "org.kustom.action.VAR_NAME";
    public static final String KUSTOM_ACTION_VAR_VALUE = "org.kustom.action.VAR_VALUE";
    public static final String KUSTOM_ACTION_VAR_NAME_ARRAY = "org.kustom.action.VAR_NAME_ARRAY";
    public static final String KUSTOM_ACTION_VAR_VALUE_ARRAY = "org.kustom.action.VAR_VALUE_ARRAY";


    /**
     * Creates an IntentService.  Invoked by your subclass's constructor.
     *
     * @param name Used to name the worker thread, important only for debugging.
     */
    public ToKservice(String name) {
        super(name);
    }

    public ToKservice() {
        super("ToKWPservice");
    }

    @Override
    protected void onHandleIntent(Intent workIntent) {


        SharedPreferences sharedpreferences = getSharedPreferences(getPackageName() + "_preferences", Context.MODE_PRIVATE);
        boolean isEnabled = sharedpreferences.getBoolean("sync", true);

        sendToKustom("rapla-active", Boolean.toString(isEnabled));

        if (!isEnabled) {
            return;
        }

        String urlString = sharedpreferences.getString("url", "");

        Log.d("qvalentin", "start toKservice");

        ConnectivityManager cm =
                (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);

        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
        boolean isConnected = activeNetwork != null &&
                activeNetwork.isConnectedOrConnecting();

        String message = getString(R.string.ToKservice_updated);


        if (isConnected) {
            boolean shouldUpdate = false;
            try {
                URL url = new URL(urlString);
                shouldUpdate = checkLastModified(url);

                if (shouldUpdate) {

                    String data = getData(url);
                    if (data != null) {
                        sendToKustom("rapla", data);
                    }
                }
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
            ;
        }
    }


    public void sendToKustom(String key, String message) {
        Log.d("qvalentin", "sendToKustom");
        Log.d("qvalentin", "sendToKustom " + message);

        Intent intent = new Intent(KUSTOM_ACTION);
        intent.putExtra(KUSTOM_ACTION_EXT_NAME, "qvalentin");
        intent.putExtra(KUSTOM_ACTION_VAR_NAME, key);
        intent.putExtra(KUSTOM_ACTION_VAR_VALUE, message);
        sendBroadcast(intent);

    }

    public String getData(URL url) {

        String result = null;
        IcalToStr icalToStr = new IcalToStr();

        try {

            SharedPreferences sharedpreferences = getSharedPreferences(getPackageName() + "_preferences", Context.MODE_PRIVATE);

            String ignore_classes = sharedpreferences.getString("ignore_classes", "");


            result = icalToStr.icalToSting(url, ignore_classes);
        } catch (Exception e) {
            Log.d("error", e.getLocalizedMessage());
        }

        return result;
    }

    public boolean checkLastModified(URL url) {

        SharedPreferences sharedpreferences = getSharedPreferences(getPackageName() + "_preferences", Context.MODE_PRIVATE);
        boolean disable_cache = sharedpreferences.getBoolean("disable_cache", true);

        if (disable_cache) {
            return disable_cache;
        }

        try {

            String lastModifiedOld = sharedpreferences.getString(getString(R.string.saved_last_modified_key), "");


            URLConnection conn = url.openConnection();
            Map<String, List<String>> map = conn.getHeaderFields();

            String headerLM = "last-modified";
            List<String> contentLength = map.get(headerLM);
            if (contentLength == null) {
                return true;
            } else {
                for (String header : contentLength) {

                    if (!header.equals(lastModifiedOld)) {
                        SharedPreferences.Editor editor = sharedpreferences.edit();
                        editor.putString(getString(R.string.saved_last_modified_key), header);
                        editor.commit();
                        return true;
                    } else return false;
                }
            }
        } catch (Exception e) {
            return true;

        }
        return true;
    }


}
