package de.qvalentin.rapla2kustom.ical;


import biweekly.ICalendar;
import biweekly.component.VEvent;
import biweekly.io.TimezoneAssignment;
import biweekly.io.TimezoneInfo;
import biweekly.io.text.ICalReader;
import biweekly.property.DateStart;
import biweekly.property.Summary;
import biweekly.util.com.google.ical.compat.javautil.DateIterator;

import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

public class IcalToStr {

/*    public static void main(String[] args) throws IOException {
        IcalToStr icalToStr=new IcalToStr();
        System.out.println(icalToStr.icalToSting(new URL("https://rapla.dhbw-karlsruhe.de/rapla?page=ical&user=braun&file=TINF19B2&day=17&month=4&year=2020&today=Heute")));
    }*/

    public IcalToStr() {
    }

    public String icalToSting(URL url, String ignore_classes) throws IOException {


        String[] classes_to_ignore = ignore_classes.split(",");


        ICalReader reader = new ICalReader(new InputStreamReader(
                url.openStream()));
        List<VeventString> veventStrings = new ArrayList<>();
        try {

            ICalendar ical;

            while ((ical = reader.readNext()) != null) {
                TimezoneInfo tzinfo = ical.getTimezoneInfo();

                VEvent event = ical.getEvents().get(0);
                DateStart dtstart = event.getDateStart();

                TimeZone timezone;
                if (tzinfo.isFloating(dtstart)) {
                    timezone = TimeZone.getDefault();
                } else {
                    TimezoneAssignment dtstartTimezone = tzinfo.getTimezone(dtstart);
                    timezone = (dtstartTimezone == null) ? TimeZone.getTimeZone("UTC") : dtstartTimezone.getTimeZone();
                }

                veventStrings.addAll(buildList(ical.getEvents(), timezone, classes_to_ignore));

            }
        } finally {
            reader.close();
        }

        Collections.sort(veventStrings, new Comparator<VeventString>() {
            public int compare(VeventString event1, VeventString event2) {
                return event1.getStartdate().compareTo(event2.getStartdate());
            }
        });
        return buidSting(veventStrings);
    }


    public List<VeventString> buildList(List<VEvent> events, TimeZone timeZone, String[] classes_to_ignore) {

        List<VeventString> eventStrings = new ArrayList<VeventString>();
        boolean repeating;
        boolean relevent;
        boolean future;
        boolean isTest;
        Date today = new Date();
        today.setHours(0);
        String category;
        Date dateStartVal;
        Date dateEndVal;
        for (VEvent event : events) {

            dateStartVal = event.getDateStart().getValue();

            Summary summary = event.getSummary();
            String summaryStr = (summary == null) ? "" : summary.getValue();

            future = dateStartVal.compareTo(new DateStart(today).getValue()) >= 0;
            category = event.getCategories().get(0).getValues().get(0).toString();
            repeating = event.getRecurrenceRule() != null;
            relevent = !category.equalsIgnoreCase("Sonstiger Termin")
                    && !Arrays.asList(classes_to_ignore).contains(summaryStr);

            if (relevent && (future || repeating)) {

                dateEndVal = event.getDateEnd().getValue();

                isTest = category.equalsIgnoreCase("Prüfung");



                DateIterator it = event.getDateIterator(timeZone);
                Date current;
                Date end;
                while (it.hasNext()) {

                    current = it.next();
                    end = new Date();
                    end.setHours(dateEndVal.getHours());
                    end.setMinutes(dateEndVal.getMinutes());
                    end.setDate(current.getDate());
                    end.setYear(current.getYear());
                    end.setMonth(current.getMonth()); // @todo: horrible, but works to set time and day separably

                    eventStrings.add(new VeventString(current, end, summaryStr, isTest));
                }
            }
        }
        return eventStrings;
    }


    public String buidSting(List<VeventString> events) {

        boolean future;
        Date today = new Date();
        boolean sameDay;
        boolean sameMonth;
        Date previousDate = today;
        boolean isTest;
        String dateEndStr;
        String dateStartStr;
        Date dateStartVal;
        Date dateEndVal;
        int counter = 0;

        DateFormat df = new SimpleDateFormat("EE: dd/MM/yyyy HH:mm", Locale.GERMANY);

        DateFormat dfEnd = new SimpleDateFormat("HH:mm");

        StringBuilder stringBuilder = new StringBuilder("Last update: " + new SimpleDateFormat("dd/MM HH:mm").format(Calendar.getInstance().getTime()));

        for (VeventString e :
                events) {

            dateStartVal = e.getStartdate();
            dateEndVal = e.getEnddate();
            future = dateEndVal.compareTo(today) >= 0;
            if (future && counter < 50) {

                sameDay = dateStartVal.getDay() == previousDate.getDay();
                sameMonth = dateStartVal.getMonth() == previousDate.getMonth();

                dateEndStr = dfEnd.format(dateEndVal);
                dateStartStr = df.format(dateStartVal);

                previousDate = dateStartVal;

                isTest = e.isTest();

                stringBuilder.append(
                        (sameDay ? "\n" : "\n[s=0.3]################################################\n[/s]") +
                                (sameMonth ? "" : "[s=0.3]################################################\n[/s]") +
                                (isTest ? "[c=testc]" : "[c=datec]") + dateStartStr + " - " + dateEndStr + (isTest ? "" : "[/c]") + ": "
                                + (isTest ? "" : "[c=namec]") + e.getDescription() + "[/c]");

                counter++;
            }
        }
        return stringBuilder.toString();
    }
}


